using System.IO;
using UnityEditor;
using UnityEditor.Animations;
using UnityEngine;

namespace Stellaxis.Omniscient.Editor
{
    /// <summary>
    /// Helper class to create an AnimatorController programmatically.
    /// </summary>
    public static class AnimatorControllerHelper
    {
        /// <summary>
        /// Creates a simple AnimatorController with a single state containing the provided animation clip.
        /// </summary>
        /// <param name="clip">The AnimationClip to assign.</param>
        /// <param name="assetPath">The asset path where the AnimatorController will be saved.</param>
        /// <returns>The created RuntimeAnimatorController.</returns>
        public static RuntimeAnimatorController CreateSimpleAnimatorController(AnimationClip clip, string assetPath)
        {
            // Ensure the asset path uses forward slashes
            assetPath = PathHelper.ToUnityAssetPath(assetPath);

            string directoryPath = PathHelper.ToUnityAssetPath(Path.GetDirectoryName(assetPath));

            if (!AssetDatabase.IsValidFolder(directoryPath))
            {
                Directory.CreateDirectory(directoryPath);
                AssetDatabase.Refresh();
            }

            // Check if AnimatorController already exists
            if (AssetDatabase.LoadAssetAtPath<AnimatorController>(assetPath) != null)
            {
                Debug.Log($"AnimatorController already exists at path: {assetPath}. Skipping creation.");
                return AssetDatabase.LoadAssetAtPath<RuntimeAnimatorController>(assetPath);
            }

            AnimatorController animatorController = AnimatorController.CreateAnimatorControllerAtPath(assetPath);

            var stateMachine = animatorController.layers[0].stateMachine;
            var state = stateMachine.AddState("Animation");
            state.motion = clip;

            AssetDatabase.SaveAssets();
            return animatorController;
        }
    }
}
