using System;
using System.Collections.Generic;

namespace Stellaxis.Omniscient.Editor
{
    #region JSON Data Classes

    /// <summary>
    /// Represents the structure of the .omni JSON file.
    /// </summary>
    [Serializable]
    public class OmniFile
    {
        public string version;
        public UnitySection unity;
        public DataSection data;
    }

    [Serializable]
    public class UnitySection
    {
        public string minimum_plugin_version;
        public string ideal_plugin_version;
    }

    [Serializable]
    public class DataSection
    {
        public CameraSection camera;
        public GeometrySection geometry;
        public VideoSection video;
    }

    [Serializable]
    public class CameraSection
    {
        public string relative_path;
        public List<CameraFrame> frames;
    }

    [Serializable]
    public class CameraFrame
    {
        public float focal_length;
        public float shutter_speed;
        public float focus_distance;
    }

    [Serializable]
    public class GeometrySection
    {
        public List<string> relative_path; // Supports multiple geometry files
    }

    [Serializable]
    public class VideoSection
    {
        public string relative_path;
        public int fps;
        public Resolution resolution;
    }

    [Serializable]
    public class Resolution
    {
        public int width;
        public int height;
    }

    #endregion
}
